﻿
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using System.Web;

class Client
{
    static string serviceNamespace;
    static string issuerKey;
    static string acsBaseAddress = "accesscontrol.windows.net";

    static void Main(string[] args)
    {
        Console.WriteLine("Enter your service namespace, then press <ENTER>");
        serviceNamespace = Console.ReadLine();

        Console.WriteLine("\nEnter your issuer key, then press <ENTER>");
        issuerKey = Console.ReadLine();

        string token = GetTokenFromACS();

        string response = SendMessageToService(token);

        Console.WriteLine("The Sample Service service returned: {0}", response);
        Console.WriteLine("Press <ENTER> to exit");
        Console.ReadLine();
    }

    private static string SendMessageToService(string token)
    {
        string message = null;

        // create the binding and address to communicate with the service
        WebHttpBinding binding = new WebHttpBinding(WebHttpSecurityMode.None);
        Uri address = new Uri(@"http://localhost/SampleService");

        WebChannelFactory<ISampleService> channelFactory = new WebChannelFactory<ISampleService>(binding, address);

        ISampleService proxy = channelFactory.CreateChannel();

        // add the token to the proxy
        using (new OperationContextScope(proxy as IContextChannel))
        {
            string authHeaderValue = string.Format("WRAP access_token=\"{0}\"", HttpUtility.UrlDecode(token));

            WebOperationContext.Current.OutgoingRequest.Headers.Add("authorization", authHeaderValue);

            // call the service and get a response
            message = proxy.SayHello();
        }

        ((IClientChannel)proxy).Close();

        channelFactory.Close();

        return message;
    }

    private static string GetTokenFromACS()
    {
        // request a token from ACS
        WebClient client = new WebClient();
        client.BaseAddress = string.Format("https://{0}.{1}", serviceNamespace, acsBaseAddress);

        NameValueCollection values = new NameValueCollection();
        values.Add("wrap_name", "SampleService");
        values.Add("wrap_password", issuerKey);
        values.Add("wrap_scope", "http://localhost/SampleService");

        byte[] responseBytes = client.UploadValues("WRAPv0.9", "POST", values);

        string response = Encoding.UTF8.GetString(responseBytes);

        Console.WriteLine("\nreceived token from ACS: {0}\n", response);

        return response
            .Split('&')
            .Single(value => value.StartsWith("wrap_access_token=", StringComparison.OrdinalIgnoreCase))
            .Split('=')[1];
    }
}

