using System;
using System.IdentityModel.Tokens;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;

using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.SecurityTokenService;
using Microsoft.IdentityModel.Tokens;


namespace WCFBook.Samples
{
    class Program
    {

        static void Main( string[] args )
        {
            X509Certificate2 stsCertificate = CertificateUtil.GetCertificate(
                StoreName.My,
                StoreLocation.LocalMachine,
                "CN=WCFSTS");
            
            // Create and setup the configuration for our STS
            SigningCredentials signingCreds = new X509SigningCredentials(stsCertificate);
            SecurityTokenServiceConfiguration config = 
                new SecurityTokenServiceConfiguration("http://WCFBookSTS", signingCreds);

            config.SecurityTokenHandlers.AddOrReplace(new CustomUsernameTokenHandler());
            // Set the STS implementation class type
            config.SecurityTokenService = typeof( MySecurityTokenService );

            // Create the WS-Trust service host with our STS configuration
            using ( WSTrustServiceHost host = new WSTrustServiceHost( config, new Uri( "http://localhost:6000" ) ) )
            {
                host.Open();
                Console.WriteLine( "WCFBookSTS started, press ENTER to stop ..." );
                Console.ReadLine();
                host.Close();
            }
        }
    }
}
