﻿namespace Wrox
{
    public class Car
    {
        public string Name { get; set; }
        public string Make { get; set; }
        public string Type { get; set; }
        public int Seats { get; set; }
    }
    [System.Runtime.Serialization.CollectionDataContract(Name = "Cars")]
    public class CarPool : List<Car>
    {
        private CarPool()
        {
            this.Add(new Car()
            {
                Name = "Dakota",
                Make = "Dodge",
                Type = "Pickup Truck",
                Seats = 8
            });
            this.Add(new Car()
            {
                Name = "TT",
                Make = "Audi",
                Type = "Sport Car",
                Seats = 2
            });
        }
        private static CarPool AllCars = null;
        public static CarPool GetCarPark()
        {
            if (AllCars == null)
                AllCars = new CarPool();
            return AllCars;
        }
    }


    [ServiceContract()]
    public interface ICarRentalService
    {
        [OperationContract]
        string HelloWorld();

        [OperationContract]
        [WebGet(UriTemplate = "/CarPool")]
        CarPool GetAllCars();

        [OperationContract]
        [WebGet(UriTemplate = "/CarPool/{carName}")]
        Car GetCar(string carName);

        [OperationContract]
        [WebGet(UriTemplate = "/CarPool/{carName}?format=xml",
        ResponseFormat = WebMessageFormat.Xml)]
        Car GetCarXML(string carName);

        [OperationContract]
        [WebGet(UriTemplate = "/CarPool/{carName}?format=json",
        ResponseFormat = WebMessageFormat.Json)]
        Car GetCarJSON(string carName);

        [OperationContract]
        [WebInvoke(UriTemplate = "CarPool/{carName}", Method = "PUT")]
        Car AddCar(string carName, Car car);

        [OperationContract]
        [WebInvoke(UriTemplate = "/CarPool/{carName}",
        Method = "DELETE")]
        void DeleteCar(string carName);
    }

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class CarRentalService : ICarRentalService
    {
        public string HelloWorld()
        {
            WebOperationContext.Current.OutgoingResponse.StatusCode =
            System.Net.HttpStatusCode.OK;
            return "Hello World";
        }

        public CarPool GetAllCars()
        {
            return CarPool.GetCarPark();
        }

        public Car GetCar(string carName)
        {
            return CarPool.GetCarPark().Find(e = > e.Name == carName);
        }

        public Car GetCarXML(string carName)
        {
            return GetCar(carName);
        }

        public Car GetCarJSON(string carName)
        {
            return GetCar(carName);
        }

        public void DeleteCar(string carName)
        {
            Car found = CarPool.GetCarPark().Find
            (e = > e.Name == carName);
                if (found == null)
                   WebOperationContext.Current.OutgoingResponse
                       .SetStatusAsNotFound();
                else
                   CarPool.GetCarPark().Remove(found);
        }

        public Car AddCar(string carName, Car car)
        {
            WebOperationContext.Current.OutgoingResponse
            .SetStatusAsCreated
            (
            new Uri("http://localhost:1234/CarPool/" + car.Name)
            );
            CarPool.GetCarPark().Add(car);
            return car;
        }
    }
}


